@@ -4,6 +4,41 @@ import json
 import pickle
 import os
 
+def ingest_poses_generic(
+    data_object,
+    pose_type
+):
+    df = convert_to_df(data_object)
+    all_column_names = df.reset_index().columns
+    if pose_type=='2d':
+        df = set_index_columns(
+            df=df,
+            index_columns='pose_2d_id'
+        )
+    elif pose_type=='3d':
+        df = set_index_columns(
+            df=df,
+            index_columns='pose_3d_id'
+        )
+    else:
+        raise ValueError('Pose type \'{}\' not recognized'.format(
+            pose_type
+        ))
+    df['timestamp'] = pd.to_datetime(df['timestamp'])
+    if 'camera_id' in all_column_names:
+        df['camera_id'] = df['camera_id'].astype('object')
+    if 'keypoint_coordinates_2d' in all_column_names:
+        df['keypoint_coordinates_2d'] = df['keypoint_coordinates_2d'].apply(convert_to_array)
+    if 'keypoint_quality_2d' in all_column_names:
+        df['keypoint_quality_2d'] = df['keypoint_quality_2d'].apply(convert_to_array)
+    if 'pose_quality_2d' in all_column_names:
+        df['pose_quality_2d'] = pd.to_numeric(df['pose_quality_2d']).astype('float')
+    if 'keypoint_coordinates_3d' in all_column_names:
+        df['keypoint_coordinates_3d'] = df['keypoint_coordinates_3d'].apply(convert_to_array)
+    if 'pose_2d_ids' in all_column_names:
+        df['pose_2d_ids'] = df['pose_2d_ids'].apply(convert_to_list)
+    return df
+
 def ingest_poses_2d(data_object):
     df = convert_to_df(data_object)
     df = set_index_columns(
@@ -159,6 +194,29 @@ def output_poses_3d_with_tracks(df, path):
             file_extension
         ))
 
+def ingest_poses_3d_with_person_ids(data_object):
+    df = convert_to_df(data_object)
+    df = set_index_columns(
+        df=df,
+        index_columns='pose_3d_id'
+    )
+    target_columns = [
+        'timestamp',
+        'keypoint_coordinates_3d',
+        'pose_2d_ids',
+        'pose_track_3d_id',
+        'person_id'
+    ]
+    if not set(target_columns).issubset(set(df.columns)):
+        raise ValueError('Data is missing fields: {}'.format(
+            set(target_columns) - set(df.columns)
+        ))
+    df = df.reindex(columns=target_columns)
+    df['timestamp'] = pd.to_datetime(df['timestamp'])
+    df['keypoint_coordinates_3d'] = df['keypoint_coordinates_3d'].apply(convert_to_array)
+    df['pose_2d_ids'] = df['pose_2d_ids'].apply(convert_to_list)
+    return df
+
 def ingest_sensor_data(
     data_object,
     id_field_names=['person_id']
@@ -296,6 +354,51 @@ def convert_to_df(data_object):
             return convert_to_df(data_deserialized)
     raise ValueError('Failed to parse data object')
 
+def convert_to_lookup_dict(data_object):
+    if isinstance(data_object, dict):
+        return data_object
+    if isinstance(data_object, pd.DataFrame):
+        if data_object.index.name is None:
+            data_object = data_object.set_index(data_object.columns[0])
+        if len(data_object.columns) < 1:
+            raise ValueError('Data object appears to tabular, but with fewer than two columns')
+        elif len(data_object.columns) == 1:
+            return data_object.iloc[:, 0].to_dict()
+        else:
+            return data_object.to_dict(orient='index')
+    if isinstance(data_object, str) and os.path.isfile(data_object):
+        file_extension = os.path.splitext(data_object)[1]
+        if len(file_extension) == 0:
+            raise ValueError('Data object appears to be a filename, but it has no extension')
+        if file_extension.lower() == '.pickle' or file_extension.lower() == '.pkl':
+            try:
+                data_deserialized = pickle.load(open(data_object, 'rb'))
+            except:
+                raise ValueError('File has extension \'pickle\' or \'pkl\', but pickle deserialization failed')
+            return convert_to_df(data_deserialized)
+        if file_extension.lower() == '.json':
+            try:
+                data_deserialized = json.load(open(data_object, 'r'))
+            except:
+                raise ValueError('File has extension \'json\', but JSON deserialization failed')
+            return convert_to_df(data_deserialized)
+        if file_extension.lower() == '.csv':
+            try:
+                data_deserialized = pd.read_csv(data_object)
+            except:
+                raise ValueError('File has extension \'csv\', but pd.read_csv(...) failed')
+            return data_deserialized
+        raise ValueError('Data object appears to be a filename, but extension \'{}\' isn\'t currently handled'.format(
+            file_extension
+        ))
+    if isinstance(data_object, str):
+            try:
+                data_deserialized = json.loads(data_object)
+            except:
+                raise ValueError('Data object is a string but it doesn\'t appear to be a valid filename or valid JSON')
+            return convert_to_df(data_deserialized)
+    raise ValueError('Failed to parse data object')
+
 def set_index_columns(
     df,
     index_columns
@@ -365,3 +468,6 @@ def nearly_equal(df_1, df_2):
     equality_df = pd.DataFrame.from_dict(equality_dict, orient='index')
     is_equal = np.all(equality_df)
     return is_equal, equality_df
+
+def convert_to_datetime_utc(datetime_object):
+    return pd.to_datetime(datetime_object, utc=True).to_pydatetime()
